import os

import click
from PIL import Image
from PIL.Image import Image as ImageFile


def save_icon(image: ImageFile, path: str, size: tuple):
    os.makedirs(os.path.dirname(path), exist_ok=True)
    new_image = image.resize(size)
    new_image.save(path)
    print(f"Icon created: '{path}'")


# icon filename to use
def generate_icons(path):
    image = Image.open(path)
    base_path = os.path.abspath(os.path.dirname(path))

    base_icon_sizes = [(16, 16), (24, 24), (32, 32), (48, 48), (64, 64)]
    linux_icon_sizes = [(128, 128), (256, 256), (512, 512), (1024, 1024)]
    mac_icon_sizes = [(128, 128), (256, 256), (512, 512), (1024, 1024)]

    # Create base icon sizes in src/main/icons/base
    for size in base_icon_sizes:
        save_icon(image, os.path.join(base_path, "base", str(size[0]) + ".png"), size)

    # Create linux icon sizes in src/main/icons/linux
    for size in linux_icon_sizes:
        save_icon(image, os.path.join(base_path, "linux", str(size[0]) + ".png"), size)

    # Create mac icon sizes in src/main/icons/mac
    for size in mac_icon_sizes:
        save_icon(image, os.path.join(base_path, "mac", str(size[0]) + ".png"), size)

    # Create Icon.ico in src/main/icons/Icon.ico
    new_logo_ico_filename = os.path.join(base_path, "Icon.ico")
    new_logo_ico = image.resize((128, 128))
    new_logo_ico.save(new_logo_ico_filename, format="ICO", quality=90)
    print("Icon created: " + new_logo_ico_filename)


@click.command()
@click.argument("path", type=click.Path(exists=False, readable=True))
def cli(path):
    generate_icons(path)


if __name__ == "__main__":
    cli()



import os

import click

from fbs_build_utils.main import get_settings, get_target_dir, update_version
from fbs_build_utils.os_cmd import execute_command
from fbs_build_utils.rename_installer import name_installer
from fbs_build_utils.generate_icons import generate_icons
from qt_build_utils import convert_ui_glob
import platform
import distro

ICON_PATH = "src/main/icons/icon.png"


@click.group()
def cli():
    pass


def generate_files():
    update_version()
    generate_icons(ICON_PATH)
    settings = get_settings()
    for ui_dir in settings["ui_paths"]:
        convert_ui_glob(ui_dir, inplace=True)


@cli.command()
def generate():
    generate_files()


@cli.command()
@click.option("--debug", is_flag=True)
@click.option("--sign", is_flag=True)
@click.option("--only-keep-renamed", is_flag=True)
def build(debug: bool, only_keep_renamed: bool, sign: bool):
    # Run the generate command
    generate_files()

    # Create EXE
    if debug:
        execute_command("fbs freeze --debug")
    else:
        execute_command("fbs freeze")

    if sign:
        execute_command("fbs sign")

    execute_command("fbs installer")

    if sign:
        execute_command("fbs sign_installer")

    info = get_settings()

    name = f"{info['app_name']}_{info['version']}"

    name_with_underscore = name.replace(" ", "_")

    if platform.system() == "Linux":
        distro_name = distro.name(pretty=True).lower().replace(" ", "_")

    elif platform.system() == "Windows":
        distro_name = "win64"

    elif platform.system() == "Darwin":
        # Check if arm or intel
        if platform.processor() == "arm":
            distro_name = "mac_arm"
        else:
            distro_name = "mac_x86"

    else:
        raise NotImplementedError(f"{platform.system()} is not supported")

    name_installer(f"{name_with_underscore}_{distro_name}", remove_source=only_keep_renamed)


@cli.command()
def run():
    # Run the app
    info = get_settings()
    portable_path = os.path.join(get_target_dir(), info["app_name"])
    execute_command(f"{portable_path}/{info['app_name']}.exe")


@cli.command()
def run_python():
    execute_command("fbs run")


@cli.command()
def run_installer():
    # Run the installer
    execute_command(os.path.join(get_target_dir(), "latest.exe"))


if __name__ == "__main__":
    cli()



import json
import os
import re

import git


def get_default_settings():
    base_path = "src/build/settings/base.default.json"
    with open(base_path, "r") as f:
        return json.load(f)


def get_target_dir():
    current_dir = os.getcwd()
    return os.path.join(current_dir, "target")


def get_settings():
    base_path = "src/build/settings/base.json"
    with open(base_path, "r") as f:
        return json.load(f)


def get_full_version():
    repo = git.Repo(search_parent_directories=True)
    return repo.git.describe("--tags", always=True, dirty="-dirty")


def get_version():
    matches = re.search(r"v(\d+\.\d+\.\d+)", get_full_version())
    if matches is not None:
        return matches.group(1)
    else:
        return "0.0.0"


def update_version():
    print("Generating setting file")
    dest_path = "src/build/settings/base.json"

    # Update the version number in the settings file
    data = get_default_settings()
    data["version"] = get_version()
    data["full_version"] = get_full_version()

    with open(dest_path, "w") as f:
        json.dump(data, f, indent=4)



from fbs_build_utils.main import get_settings
import os
import shutil
import platform


def name_installer(target_name: str, remove_source=False):
    data = get_settings()

    # Rename the installer
    if platform.system() == "Windows":
        source_file = f"target/{data['app_name']}Setup.exe"
        # dest_file = f"target/{data['app_name']}_{data['version']}_Installer_win64.exe"
        dest_file = f"target/{target_name}.exe"
    elif platform.system() == "Darwin":
        source_file = f"target/{data['app_name']}.dmg"
        dest_file = f"target/{target_name}.dmg"
    elif platform.system() == "Linux":
        source_file = f"target/{data['app_name']}.deb"
        dest_file = f"target/{target_name}.deb"

    else:
        raise NotImplementedError(f"{platform.system()} is not supported")

    # Remove the old installer if it exists
    if os.path.exists(dest_file):
        os.remove(dest_file)

    # Rename the installer
    shutil.copyfile(source_file, dest_file)

    if remove_source:
        os.remove(source_file)


def package_portable(target_name: str):
    data = get_settings()
    source_path = f"target/{data['app_name']}"
    target_path = f"target/{target_name}"
    # Create zip
    shutil.make_archive(target_path, "zip", source_path)



import os
import subprocess


# from icons.icongenerator import generate_icons  # noqa: E402
from qt_build_utils.path_manager import manager


def execute_command(cmd: str):
    if not manager.is_path_setup():
        manager.is_path_setup()

    # Execute a command and print to stdout and stderr
    print(f"Executing command: '{cmd}'")

    proc = subprocess.Popen(cmd, shell=True, cwd=os.getcwd())

    # Check for erros
    if proc.wait() != 0:
        raise RuntimeError(f"Command {cmd} failed with error code {proc.returncode}")



from .cli import cli
import sys
import os

if __name__ == "__main__":
    # Add exectuables scripts folder to path
    python_path = os.path.dirname(sys.executable)

    # Insert the path to the beginning of the path
    sys.path.insert(0, python_path)

    cli()
